@@ -1,201 +1,161 @@
 from pytest import raises
+from chomsky import ParseException
 from plywood import (
-    Plywood, PlywoodValue,
-    PlywoodVariable, PlywoodString, PlywoodNumber,
-    PlywoodOperator, PlywoodUnaryOperator,
-    PlywoodParens, PlywoodList, PlywoodDict,
-    PlywoodKvp, PlywoodFunction, PlywoodBlock,
-    ParseException,
+    plywood, Plywood, PlywoodVariable,
+    )
+from . import (
+    assert_number,
+    assert_string,
+    assert_variable,
+    assert_unary,
+    assert_operator,
+    assert_function,
+    assert_kvp,
+    assert_block,
+    assert_list,
+    assert_slice,
+    assert_parens,
+    assert_dict,
     )
 
 
-def assert_number(test, value):
-    assert isinstance(test, PlywoodNumber)
-    assert isinstance(test.value, type(value))
-    assert test.value == value
-
-
-def assert_string(test, value):
-    assert isinstance(test, PlywoodString)
-    assert test.value == value
-
-
-def assert_variable(test, name):
-    assert isinstance(test, PlywoodVariable)
-    assert test.name == name
-
-
-def assert_unary(test, op):
-    assert isinstance(test, PlywoodUnaryOperator)
-    assert test.operator == op
-
-
-def assert_operator(test, op):
-    assert isinstance(test, PlywoodOperator)
-    assert test.operator == op
-
-
-def assert_function(test, name=None):
-    assert isinstance(test, PlywoodFunction)
-    if name:
-        assert_variable(test.left, name)
-
-
-def assert_kvp(test, key):
-    assert isinstance(test, PlywoodKvp)
-    if key:
-        if not isinstance(key, PlywoodValue):
-            assert_string(test.key, key)
-        else:
-            assert test.key == key
-
-
-def assert_block(test, count=None):
-    assert isinstance(test, PlywoodBlock)
-    if count is not None:
-        assert len(test.lines) == count
-
-
-def assert_parens(test, arg_count=None, kwarg_count=0):
-    assert isinstance(test, PlywoodParens)
-    if arg_count is not None:
-        assert len(test.args) == arg_count
-        assert len(test.kwargs) == kwarg_count
-
-
-def assert_dict(test, count=None):
-    assert isinstance(test, PlywoodDict)
-    if count is not None:
-        assert len(test.values) == count
-
-
 def test_variable():
-    test = Plywood('foo').parse()[0]
+    test = Plywood('foo').compile()[0]
     assert_variable(test, 'foo')
 
 
+def test_variable_dashes():
+    test = Plywood('foo-bar').compile()[0]
+    assert_variable(test, 'foo-bar')
+
+
+def test_variable_colons():
+    test = Plywood('foo:bar').compile()[0]
+    assert_variable(test, 'foo:bar')
+
+
 def test_blank():
-    assert [] == Plywood('# comment').parse()
-    assert [] == Plywood('\n\n# comment\n\n').parse()
-    assert [] == Plywood('\n\n# comment\n # \n').parse()
-    assert [] == Plywood('\n\n# comment\n # \n#').parse()
-    assert [] == Plywood('\n\n# comment\n   \n#').parse()
+    assert [] == Plywood('# comment').compile()
+    assert [] == Plywood('\n\n# comment\n\n').compile()
+    assert [] == Plywood('\n\n# comment\n # \n').compile()
+    assert [] == Plywood('\n\n# comment\n # \n#').compile()
+    assert [] == Plywood('\n\n# comment\n   \n#').compile()
 
 
 def test_comment_with_variable():
-    test = Plywood('foo # comment').parse()[0]
+    test = Plywood('foo # comment').compile()[0]
     assert_variable(test, 'foo')
 
-    test = Plywood('foo # comment\nbar # comment').parse()[0]
+    test = Plywood('foo # comment\nbar # comment').compile()[0]
     assert_variable(test, 'foo')
 
-    test = Plywood('\n# comment1\nfoo # comment2\n\n\n  # comment3').parse()[0]
+    test = Plywood('\n# comment1\nfoo # comment2\n\n\n  # comment3').compile()[0]
     assert_variable(test, 'foo')
 
 
 def test_string():
-    test = Plywood('"foo"').parse()[0]
+    test = Plywood('"foo"').compile()[0]
     assert_string(test, 'foo')
 
 
 def test_integer():
-    test = Plywood('1').parse()[0]
+    test = Plywood('1').compile()[0]
     assert_number(test, 1)
 
 
 def test_neg_integer():
-    test = Plywood('-1').parse()[0]
+    test = Plywood('-1').compile()[0]
     assert_number(test, -1)
 
 
 def test_float():
-    test = Plywood('0.123').parse()[0]
+    test = Plywood('0.123').compile()[0]
     assert_number(test, 0.123)
 
-    test = Plywood('1.123').parse()[0]
+    test = Plywood('1.123').compile()[0]
     assert_number(test, 1.123)
 
 
 def test_neg_float():
-    test = Plywood('-0.123').parse()[0]
+    test = Plywood('-0.123').compile()[0]
     assert_number(test, -0.123)
 
 
 def test_hexadecimal():
-    test = Plywood('0x1e32').parse()[0]
+    test = Plywood('0x1e32').compile()[0]
     assert_number(test, 7730)
 
-    test = Plywood('0x0001ea').parse()[0]
+    test = Plywood('0x0001ea').compile()[0]
     assert_number(test, 490)
 
-    test = Plywood('0x0').parse()[0]
+    test = Plywood('0x0').compile()[0]
     assert_number(test, 0)
 
-    test = Plywood('0xFFFF').parse()[0]
+    test = Plywood('0xFFFF').compile()[0]
     assert_number(test, 65535)
 
 
 def test_octal():
-    test = Plywood('0o716').parse()[0]
+    test = Plywood('0o716').compile()[0]
     assert_number(test, 462)
 
-    test = Plywood('0o0716').parse()[0]
+    test = Plywood('0o0716').compile()[0]
     assert_number(test, 462)
 
 
 def test_binary():
-    test = Plywood('0b010101').parse()[0]
+    test = Plywood('0b010101').compile()[0]
     assert_number(test, 21)
 
 
 def test_neg_hexadecimal():
-    test = Plywood('-0x1e32').parse()[0]
+    test = Plywood('-0x1e32').compile()[0]
     assert_number(test, -7730)
 
 
 def test_unary():
-    test = Plywood('-bar').parse()[0]
+    test = Plywood('-bar').compile()[0]
     assert_unary(test, '-')
     assert_variable(test.value, 'bar')
 
 
 def test_addition():
-    test = Plywood('foo + bar').parse()[0]
+    test = Plywood('foo + bar').compile()[0]
     assert_operator(test, '+')
     assert_variable(test.left, 'foo')
     assert_variable(test.right, 'bar')
 
 
 def test_multiplication():
-    test = Plywood('foo * bar').parse()[0]
+    test = Plywood('foo * bar').compile()[0]
     assert_operator(test, '*')
     assert_variable(test.left, 'foo')
     assert_variable(test.right, 'bar')
 
 
 def test_add_mult_precedence():
-    test = Plywood('foo + bar * baz').parse()[0]
+    test = Plywood('foo + bar * baz').compile()[0]
     assert_operator(test, '+')
     assert_variable(test.left, 'foo')
     assert_operator(test.right, '*')
     assert_variable(test.right.left, 'bar')
     assert_variable(test.right.right, 'baz')
 
-    test = Plywood('foo * bar + baz').parse()[0]
+    test = Plywood('foo * bar + baz').compile()[0]
     assert_operator(test, '+')
     assert_variable(test.right, 'baz')
     assert_operator(test.left, '*')
     assert_variable(test.left.left, 'foo')
     assert_variable(test.left.right, 'bar')
 
-    test = Plywood('foo + bar + baz').parse()[0]
+    test = Plywood('foo + bar + baz').compile()[0]
     assert_operator(test, '+')
     assert_variable(test.right, 'baz')
     assert_operator(test.left, '+')
     assert_variable(test.left.left, 'foo')
     assert_variable(test.left.right, 'bar')
 
-    test = Plywood('foo * bar + baz * foo').parse()[0]
+    test = Plywood('foo * bar + baz * foo').compile()[0]
     assert_operator(test, '+')
     assert_operator(test.left, '*')
     assert_variable(test.left.left, 'foo')
@@ -206,14 +166,14 @@ def test_add_mult_precedence():
 
 
 def test_unary_precedence():
-    test = Plywood('-~bar').parse()[0]
+    test = Plywood('-~bar').compile()[0]
     assert_unary(test, '-')
     assert_unary(test.value, '~')
     assert_variable(test.value.value, 'bar')
 
 
 def test_precedence():
-    test = Plywood('1 < 2 and 2 > 1 and 2+2 == 4').parse()[0]
+    test = Plywood('1 < 2 and 2 > 1 and 2+2 == 4').compile()[0]
     assert_operator(test, 'and')
     assert_operator(test.left, 'and')
     assert_operator(test.left.left, '<')
@@ -225,28 +185,28 @@ def test_precedence():
     assert_number(test.right.left.right, 2)
     assert_number(test.right.right, 4)
 
-    test = Plywood('2*3**4').parse()[0]
+    test = Plywood('2*3**4').compile()[0]
     assert_operator(test, '*')
     assert_number(test.left, 2)
     assert_operator(test.right, '**')
     assert_number(test.right.left, 3)
     assert_number(test.right.right, 4)
 
-    test = Plywood('2**3*4').parse()[0]
+    test = Plywood('2**3*4').compile()[0]
     assert_operator(test, '*')
     assert_operator(test.left, '**')
     assert_number(test.left.left, 2)
     assert_number(test.left.right, 3)
     assert_number(test.right, 4)
 
-    test = Plywood('2**3**4').parse()[0]
+    test = Plywood('2**3**4').compile()[0]
     assert_operator(test, '**')
     assert_number(test.left, 2)
     assert_operator(test.right, '**')
     assert_number(test.right.left, 3)
     assert_number(test.right.right, 4)
 
-    test = Plywood('-foo + ~bar').parse()[0]
+    test = Plywood('-foo + ~bar').compile()[0]
     assert_operator(test, '+')
     assert_unary(test.left, '-')
     assert_variable(test.left.value, 'foo')
@@ -255,7 +215,7 @@ def test_precedence():
 
 
 def test_parens():
-    test = Plywood('(-foo + ~bar)').parse()[0]
+    test = Plywood('(-foo + ~bar)').compile()[0]
     assert_parens(test, 1)
 
     assert_operator(test.args[0], '+')
@@ -266,7 +226,7 @@ def test_parens():
 
 
 def test_parens_two():
-    test = Plywood('(a, b)').parse()[0]
+    test = Plywood('(a, b)').compile()[0]
     assert_parens(test, 2)
 
     assert_variable(test.args[0], 'a')
@@ -274,12 +234,12 @@ def test_parens_two():
 
 
 def test_args_two():
-    test = Plywood('foo a, b').parse()[0]
+    test = Plywood('foo a, b').compile()[0]
     assert_function(test, 'foo')
 
 
 def test_args_three():
-    test = Plywood('foo a, b, c').parse()[0]
+    test = Plywood('foo a, b, c').compile()[0]
     assert_function(test, 'foo')
 
     assert_variable(test.right.args[0], 'a')
@@ -288,7 +248,7 @@ def test_args_three():
 
 
 def test_args_three_kwarg():
-    test = Plywood('foo a, b, d=e, c, f=g').parse()[0]
+    test = Plywood('foo a, b, d=e, c, f=g').compile()[0]
     assert_function(test, 'foo')
 
     assert_parens(test.right, 3, 2)
@@ -306,7 +266,7 @@ def test_parens_multiline():
     a,
     b,
     c,
-)''').parse()[0]
+)''').compile()[0]
     assert_parens(test, 3)
 
     assert_variable(test.args[0], 'a')
@@ -319,7 +279,7 @@ def test_parens_args():
     a,
     b,
     c=d,
-)''').parse()[0]
+)''').compile()[0]
     assert_parens(test, 2, 1)
 
     assert_variable(test.args[0], 'a')
@@ -329,21 +289,28 @@ def test_parens_args():
 
 
 def test_list():
-    test = Plywood('[1, "two", three, (four) + -five]').parse()[0]
-    assert isinstance(test, PlywoodList)
+    test = Plywood('[1, "two", three, (four) + -five]').compile()[0]
+    assert_list(test, 4)
 
     assert_number(test.values[0], 1)
     assert_string(test.values[1], 'two')
     assert_variable(test.values[2], 'three')
     assert_operator(test.values[3], '+')
-    assert isinstance(test.values[3].left, PlywoodParens)
+    assert_parens(test.values[3].left, 1)
     assert_variable(test.values[3].left.args[0], 'four')
     assert_unary(test.values[3].right, '-')
     assert_variable(test.values[3].right.value, 'five')
 
 
+def test_list_trailing_comma():
+    test = Plywood('[1, 2, ]').compile()[0]
+    assert_list(test, 2)
+    assert_number(test.values[0], 1)
+    assert_number(test.values[1], 2)
+
+
 def test_assign():
-    test = Plywood('a = b = c + 1').parse()[0]
+    test = Plywood('a = b = c + 1').compile()[0]
 
     assert_operator(test, '=')
     assert_variable(test.left, 'a')
@@ -355,7 +322,7 @@ def test_assign():
 
 
 def test_function():
-    test = Plywood('foo(bar, baz)').parse()[0]
+    test = Plywood('foo(bar, baz)').compile()[0]
     assert_function(test, 'foo')
     assert_parens(test.right, 2)
 
@@ -364,7 +331,7 @@ def test_function():
 
 
 def test_function_kwargs():
-    test = Plywood('foo(bar, 1, key="value")').parse()[0]
+    test = Plywood('foo(bar, 1, key="value")').compile()[0]
     assert_function(test, 'foo')
     assert_variable(test.left, 'foo')
     assert_parens(test.right, 2, 1)
@@ -375,7 +342,7 @@ def test_function_kwargs():
 
 
 def test_autofunction():
-    test = Plywood('foo bar, baz(a b), c=d').parse()[0]
+    test = Plywood('foo bar, baz(a b), c=d').compile()[0]
     assert_function(test, 'foo')
 
     assert_variable(test.right.args[0], 'bar')
@@ -385,18 +352,18 @@ def test_autofunction():
 
 
 def test_dict():
-    test = Plywood('{"foo": bar}').parse()[0]
+    test = Plywood('{"foo": bar}').compile()[0]
     assert_dict(test, 1)
     assert_kvp(test.values[0], 'foo')
     assert_variable(test.values[0].value, 'bar')
 
 
 def test_dict_two():
-    test = Plywood('{"foo": bar, bar: baz}').parse()[0]
+    test = Plywood('{"foo": bar, bar: baz}').compile()[0]
     assert_dict(test, 2)
     assert_kvp(test.values[0], 'foo')
     assert_variable(test.values[0].value, 'bar')
-    assert_kvp(test.values[1], PlywoodVariable('bar'))
+    assert_kvp(test.values[1], PlywoodVariable(-1, 'bar'))
     assert_variable(test.values[1].value, 'baz')
 
 
@@ -404,16 +371,16 @@ def test_dict_multiline():
     test = Plywood('''{
         "foo": bar,
         "bar": baz
-        }''').parse()[0]
+        }''').compile()[0]
     assert_dict(test, 2)
     assert_kvp(test.values[0], 'foo')
     assert_variable(test.values[0].value, 'bar')
-    assert_kvp(test.values[1], PlywoodVariable('bar'))
+    assert_kvp(test.values[1], PlywoodVariable(-1, 'bar'))
     assert_variable(test.values[1].value, 'baz')
 
 
 def test_multiline_commands():
-    test = Plywood("foo\nbar").parse()
+    test = Plywood("foo\nbar").compile()
     assert_variable(test[0], 'foo')
     assert_variable(test[1], 'bar')
 
@@ -426,7 +393,7 @@ foo
 123
 bar
 foo(bar, item='value')
-""").parse()
+""").compile()
     assert_variable(test[0], 'foo')
     assert_string(test[1], 'foo')
     assert_number(test[2], 1)
@@ -437,7 +404,7 @@ foo(bar, item='value')
 
 def test_block_indent_fail():
     with raises(ParseException):
-        test = Plywood('   foo').parse()
+        test = Plywood('   foo').compile()
         assert test == None
 
 
@@ -445,7 +412,7 @@ def test_block():
     test = Plywood('''
 if True or False:
     print "True!"
-''').parse()
+''').compile()
 
     assert_block(test, 1)
     assert_function(test[0], 'if')
@@ -465,7 +432,7 @@ def test_inline_block():
     """
     test = Plywood('''
 if True or False:  print "True!"
-''').parse()
+''').compile()
 
     assert_block(test, 1)
     assert_function(test[0], 'if')
@@ -482,7 +449,7 @@ if True or False:  print "True!"
 def test_happy_fun_block():
     test = Plywood('''
 div: p: 'text'
-''').parse()
+''').compile()
 
     assert_block(test, 1)
     assert_function(test[0], 'div')
@@ -495,7 +462,7 @@ div: p: 'text'
 def test_classy_div():
     test = Plywood('''
 div.classy: 'text'
-''').parse()
+''').compile()
 
     assert_block(test, 1)
     assert_function(test[0])
@@ -506,10 +473,61 @@ div.classy: 'text'
     assert_string(test[0].block[0], 'text')
 
 
+def test_getitem():
+    test = Plywood('''
+div['something'] = 'text'
+''').compile()
+
+    assert_block(test, 1)
+    assert_operator(test[0], '=')
+    assert_operator(test[0].left, '[]')
+    assert_variable(test[0].left.left, 'div')
+    assert_slice(test[0].left.right, 1)
+    assert_string(test[0].left.right[0], 'something')
+    assert_string(test[0].right, 'text')
+
+
+def test_getitem_2():
+    test = Plywood('''
+div['something']['else'] = 'text'
+''').compile()
+
+    assert_block(test, 1)
+    assert_operator(test[0], '=')
+    assert_operator(test[0].left, '[]')
+    assert_operator(test[0].left.left, '[]')
+    assert_variable(test[0].left.left.left, 'div')
+    assert_slice(test[0].left.left.right, 1)
+    assert_string(test[0].left.left.right[0], 'something')
+
+    assert_slice(test[0].left.right, 1)
+    assert_string(test[0].left.right[0], 'else')
+    assert_string(test[0].right, 'text')
+
+
+def test_getitem_getattr():
+    test = Plywood('''
+div['item'].attr['attritem'] = 'text'
+''').compile()
+
+    assert_block(test, 1)
+    print test[0]
+    assert_operator(test[0], '=')
+    assert_operator(test[0].left, '[]')
+    assert_operator(test[0].left.left, '.')
+    assert_operator(test[0].left.left.left, '[]')
+    assert_variable(test[0].left.left.left.left, 'div')
+    assert_slice(test[0].left.left.left.right, 1)
+    assert_string(test[0].left.left.left.right.values[0], 'item')
+    assert_slice(test[0].left.right, 1)
+    assert_string(test[0].left.right[0], 'attritem')
+    assert_string(test[0].right, 'text')
+
+
 def test_id_div():
     test = Plywood('''
 div@any_id: 'text'
-''').parse()
+''').compile()
 
     assert_block(test, 1)
     assert_function(test[0])
@@ -523,7 +541,7 @@ div@any_id: 'text'
 def test_classy_id_div():
     test = Plywood('''
 div.classy@any_id: 'text'
-''').parse()
+''').compile()
 
     assert_block(test, 1)
     assert_function(test[0])
@@ -540,7 +558,7 @@ def test_multi_inline():
     test = Plywood('''
 foo: bar
 foo: bar
-''').parse()
+''').compile()
     assert_block(test, 2)
     assert_function(test[0], 'foo')
     assert_block(test[0].block, 1)
@@ -556,7 +574,7 @@ foo:
 foo:
 foo:
 foo:
-''').parse()
+''').compile()
     assert_block(test, 4)
     assert_function(test[0], 'foo')
     assert_block(test[0].block, 0)
@@ -574,7 +592,7 @@ foo:
     foo:
         foo:
             foo:
-''').parse()
+''').compile()
     assert_block(test, 1)
     assert_function(test[0], 'foo')
     assert_block(test[0].block, 1)
@@ -592,7 +610,7 @@ foo:
     foo:
         BAH:  bar
         foo:  bar
-''').parse()
+''').compile()
     assert_block(test, 1)
     assert_function(test[0], 'foo')
     assert_block(test[0].block, 1)
@@ -608,13 +626,13 @@ foo:
 
 def test_multi_mixed_fail():
     with raises(ParseException):
-        Plywood('''
+        plywood('''
 foo:
     foo:
         BAH:  bar
         # too much indent:
           foo:  bar
-''').parse()
+''').compile()
         assert False
 
 
@@ -624,7 +642,7 @@ foo:
     foo
 BAH:
     BAH
-''').parse()
+''').compile()
     assert_block(test, 2)
     assert_function(test[0], 'foo')
     assert_block(test[0].block, 1)
@@ -642,7 +660,7 @@ BAH: BAH
 bar:
     baz
 baz: 'baz'
-''').parse()
+''').compile()
     assert_block(test, 4)
     assert_function(test[0], 'foo')
     assert_block(test[0].block, 1)
@@ -659,3 +677,51 @@ baz: 'baz'
     assert_function(test[3], 'baz')
     assert_block(test[3].block, 1)
     assert_string(test[3].block[0], 'baz')
+
+
+def test_mixed_block_indented_last():
+    test = Plywood('''
+foo: bar:
+        baz
+''').compile()
+    assert_block(test, 1)
+    assert_function(test[0], 'foo')
+    assert_block(test[0].block, 1)
+    assert_block(test[0].block[0].block, 1)
+
+
+def test_mixed_block_fail():
+    with raises(ParseException):
+        test = Plywood('''
+foo:
+    bar:
+        baz
+      baz
+''').compile()
+        assert test == None
+
+
+def test_for_syntax():
+    test = Plywood('''
+for value in [1,2,3]:
+    value
+''').compile()
+    assert_block(test, 1)
+    assert_function(test[0], 'for')
+    assert_parens(test[0].right, 1, 0)
+    assert_operator(test[0].right.args[0], 'in')
+    assert_variable(test[0].right.args[0].left, 'value')
+    assert_list(test[0].right.args[0].right, 3)
+    assert_block(test[0].block, 1)
+    assert_variable(test[0].block[0], 'value')
+
+
+def test_attr_call_precedence():
+    test = Plywood('''
+a.b()
+''').compile()
+    assert_block(test, 1)
+    assert_function(test[0])
+    assert_operator(test[0].left, '.')
+    assert_variable(test[0].left.left, 'a')
+    assert_variable(test[0].left.right, 'b')
